/*
 * File:   MotorControl.c
 * Author: acain
 *
 * Created on May 8, 2016, 4:47 PM
 */

// Include files
#include <xc.h>
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "PIC16F1788.h"
#include "MotorControl.h"
#include <stdio.h>
#include "bitdefs.h"
#include "PairingSM.h"

// Define constants
#define PWM_FREQ 7500 
#define PR2_VAL (8000000/PWM_FREQ) / (4*4) - 1
#define LSB8_MASK 0xff
#define LSB2_MASK 0x03
#define BITS_5and4_MASK 0x30 
#define FORWARD = 0x01
#define BACKWARD = 0x00

// Declare module-level variables
static uint8_t MyPriority;
static uint8_t MotorSpeed;

static int8_t DriveCommand;

// Declare module-level prototypes
static void InitPWM(void);
static void GetSpeed(void);
static void InitOtherPins(void);

/****************************************************************************
 Function
     InitMC

 Parameters
     uint8_t : the priority of this service

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Saves away priority and starts service
 Notes

 Author
    Glenn Cocoa
****************************************************************************/
bool InitMC ( uint8_t Priority )
{
  MyPriority = Priority;
  // Initialize PWM hardware
  InitPWM();
  // Init other pins
  InitOtherPins();
  // Get into run function by posting ES_INIT event
  ES_Event ThisEvent;
  ThisEvent.EventType = ES_INIT;
  PostMC(ThisEvent);
  return true;
}


/****************************************************************************
 Function
     PostMC

 Parameters
     ES_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the post operation failed, True otherwise

 Description
     Posts an event to this service's queue
 Notes

 Author
       Glenn Cocoa
****************************************************************************/
bool PostMC( ES_Event ThisEvent ) 
{
    return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
     RunMC

 Parameters
     ES_Event: the event to process

 Returns
     ES_Event: an event to return

 Description
     the run function for the service 
 Notes
     
 Author
       Glenn Cocoa
****************************************************************************/
ES_Event RunMC( ES_Event CurrentEvent )
{
    ES_Event ReturnEvent;
    ReturnEvent.EventType = ES_NO_EVENT; // assume no errors 
    static int16_t RightDuty;
    static int16_t LeftDuty;
    static int16_t RightDuty2Register;
    static int16_t LeftDuty2Register;
    if(CurrentEvent.EventType == ES_INIT){
        //ES_Timer_InitTimer(MC_TIMER,10);
    } else if (CurrentEvent.EventType == ES_DRIVE_COMMAND){
        RightDuty = 100*getDriveByte()/128;
        LeftDuty = 100*getDriveByte()/128;
        RightDuty -= (50*getTurnByte()/128);
        LeftDuty += (50*getTurnByte()/128);
        if (RightDuty > 99){
            RightDuty = 99;
        }
        else if (RightDuty < -99){
            RightDuty = -99;
        }
        if (LeftDuty > 99){
            LeftDuty = 99;
        }
        else if (LeftDuty < -99){
            LeftDuty = -99;
        }
        RightDuty2Register = RightDuty*4*(PR2_VAL+1)/100;   // This values goes from -MAX to MAX, MAX = 4*(PR2_VAL+1))
        LeftDuty2Register = LeftDuty*4*(PR2_VAL+1)/100;     // This values goes from -MAX to MAX, MAX = 4*(PR2_VAL+1))
        //Take care of right motor
        if (RightDuty2Register < 0) {
            RightDuty2Register = (100+RightDuty)*4*(PR2_VAL+1)/100; // This would make it positive and inverts the polarity
            LATC2 = 1;
        } else {
            LATC2 = 0;
        }
        CCPR2L = (RightDuty2Register >> 2) & LSB8_MASK; // mask bits 2-9 of 16-bit integer before writing to 8-bit register
        CCP2CON |= (((RightDuty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK);
        //Take care of left motor
        if (LeftDuty2Register < 0) {
            LeftDuty2Register = (100+LeftDuty)*4*(PR2_VAL+1)/100; // This would make it positive and inverts the polarity
            LATC5 = 1;
        } else {
            LATC5 = 0;
        }
        CCPR3L = (LeftDuty2Register >> 2) & LSB8_MASK; // mask bits 2-9 of 16-bit integer before writing to 8-bit register
        CCP3CON |= (((LeftDuty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK);
        
//        if (CurrentEvent.EventParam == 0x01){ // FORWARD
//            //LATA5 = 1; // debug
//            LATC2 = 0;
//            LATC5 = 0;
//            //DutyCycle = 0; // debug
//            DutyCycle = 100*getDriveRight()/127;
//            Duty2Register = DutyCycle*4*(PR2_VAL+1)/100;
//            CCPR3L = (Duty2Register >> 2) & LSB8_MASK; // mask lower 8 bits of 16-bit integer before writing to 8-bit register
//            CCP3CON |= (((Duty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK);
//            // set other motor's duty cycle
//            //DutyCycle = 0; // debug
//            DutyCycle = 100*getDriveLeft()/127;
//            Duty2Register = DutyCycle*4*(PR2_VAL+1)/100;
//            CCPR2L = (Duty2Register >> 2) & LSB8_MASK;
//            CCP2CON |= (((Duty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK);
//        } else if (CurrentEvent.EventParam == 0x00){ // BACKWARD
//            //LATA5 = 0; // debug
//            LATC2 = 1;
//            LATC5 = 1;
//            //DutyCycle = 100; // debug
//            DutyCycle = 100*(127 - getDriveLeft())/127;
//            Duty2Register = DutyCycle*4*(PR2_VAL+1)/100;
//            CCPR3L = (Duty2Register >> 2) & LSB8_MASK; // mask lower 8 bits of 16-bit integer before writing to 8-bit register
//            CCP3CON |= (((Duty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK);
//            // set other motor's duty cycle
//            //DutyCycle = 100; // debug
//            DutyCycle = 100*(127 - getDriveRight())/127;
//            Duty2Register = DutyCycle*4*(PR2_VAL+1)/100;
//            CCPR2L = (Duty2Register >> 2) & LSB8_MASK;
//            CCP2CON |= (((Duty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK);
//        }
    } else if (CurrentEvent.EventType == ES_TIMEOUT
            && CurrentEvent.EventParam == MC_TIMER){
//        // Update PWM duty cycle
//        GetSpeed();
//        DutyCycle = 100*MotorSpeed/7;
//        Duty2Register = DutyCycle*4*(PR2_VAL+1)/100;
//        CCPR3L = (Duty2Register >> 2) & LSB8_MASK; // mask lower 8 bits of 16-bit integer before writing to 8-bit register
//        CCP3CON |= ((Duty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK;
//        CCPR2L = (Duty2Register >> 2) & LSB8_MASK;
//        CCP2CON |= ((Duty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK;
//        
//        // Start timer
//        ES_Timer_InitTimer(MC_TIMER,10);
//    } else if (CurrentEvent.EventType == ES_Fan1){
//            MotorSpeed = CurrentEvent.EventParam;
//            DutyCycle = 100*MotorSpeed/7;
//            Duty2Register = DutyCycle*4*(PR2_VAL+1)/100;
//            CCPR3L = (Duty2Register >> 2) & LSB8_MASK; // mask lower 8 bits of 16-bit integer before writing to 8-bit register
//            CCP3CON |= ((Duty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK;
//        
//    } else if (CurrentEvent.EventType == ES_Fan2){
//            MotorSpeed = CurrentEvent.EventParam;
//            DutyCycle = 100*MotorSpeed/7;
//            Duty2Register = DutyCycle*4*(PR2_VAL+1)/100;
//            CCPR2L = (Duty2Register >> 2) & LSB8_MASK;
//            CCP2CON |= ((Duty2Register & (LSB2_MASK)) << 4) & BITS_5and4_MASK;
//      
    }else if (CurrentEvent.EventType == ES_LED){
        if (CurrentEvent.EventParam == 0x01){
            //turn on LED
            LATA1 = 1;
        }else if (CurrentEvent.EventParam == 0x00){
            //turn off LED
            LATA1 = 0;
        }
    }else if (CurrentEvent.EventType == ES_LiftFan){
        if (CurrentEvent.EventParam == 0x01){
            //turn on LiftFan
            LATA0 = 1;
        }else if (CurrentEvent.EventParam == 0x00){
            //turn off LiftFan
            LATA0 = 0;
        }
    }
    return ReturnEvent;
}

/*****************************Helper Functions******************************/
// InitPWM: function to initialize PWM for drive motors
static void InitPWM(){
    /* Notes:
     * CCP3 resets to PC6
     * CCP2 resets to PC1
     */
    
    // Disable CCP1 and CCP2 by setting TRISC1 and TRISC2
    TRISC1 = 0x01;
    TRISC6 = 0x01;
    
    // Load PR2 register with PWM period
    PR2 = PR2_VAL;
    
    // Configure CCP Modules for PWM mode 
    CCP3M0 = 0x00;
    CCP3M1 = 0x00;
    CCP3M2 = 0x01;
    CCP3M3 = 0x01;
    
    CCP2M0 = 0x00;
    CCP2M1 = 0x00;
    CCP2M2 = 0x01;
    CCP2M3 = 0x01;
    
    // Load CCPRxL registers and CCPxCON registers with PWM duty cycle
    CCPR3L = 0x00;//0x0c;
    DC3B0 = 0x00;
    DC3B1 = 0x00; //0x01;
    
    CCPR2L = 0x00;//0x0c;
    DC2B0 = 0x00;
    DC2B1 = 0x00;//0x01;
    
    /* Configure and start Timer2:
     * Clear TMR2IF interrupt flag
     * Config T2CKPS bits of T2CON with prescaler
     * Enable Timer2 by setting TMR2ON in T2CON
     */
    TMR2IF = 0x00;
    T2CKPS0 = 0x00;
    T2CKPS1 = 0x01;
    TMR2ON = 0x01;
    
    // Enable CCP1 and CCP2 via TRISC
    TRISC1 = 0x00;
    TRISC6 = 0x00;
    
    // Enable PC2 and PC5
    TRISC2 = 0x00;
    TRISC5 = 0x00;
    
    /* Other important enables:
     * global ints
     * peripheral ints
     * CCP1 int
     * CCP2 int
     */
    //CCP1IE = 0x01;
    //CCP2IE = 0x01;
    //PEIE = 0x01;
    //GIE = 0x01;
}

// GetSpeed: function to decide PWM duty cycle based on input pins
// input pins: C4, C5, C7 (8 different possible speeds)
static void GetSpeed(){
    MotorSpeed = 0x00;
    if(RC4 == 0x01){
        MotorSpeed |= BIT0HI;
        // Debug: toggle A2 
        //LATA2 ^= 1;
    }
    if(RC5 == 0x01){
        MotorSpeed |= BIT1HI;
    }
    if(RC7 == 0x01){
        MotorSpeed |= BIT2HI;
    }
}

static void InitOtherPins(){
    LATA0 = 0;
}